#!/bin/bash
# version 1.0 June 24, 2016
# This script is using the AWS cli.
# It assumes that the aws command is part of the search path
AMI=$1
PEM=$2
INSTANCETYPE=$3
IP=$4
CIDR=$5
SGNAME=$6
NAMETAG=$7
case $1 in
        -h | -help)p
        echo "Use this command with the following options:"
        echo "$0 -h                : to obtain this output"
        echo "$0 -help             : to obtain this output"
        echo "$0                   : enter information through a dialog"
        echo "$0 ami-id pem-name instance-type ip-address cidr security-group \"name tag\" "
	echo "Example:"
	echo " ./createHana.sh ami-6b5a5601 myPEM r3.4xlarge 10.79.7.96 10.79.7.0/24 SAP_LL_HS \"my-HANA-System96\"" 
        exit
        ;;
esac    
if [[ -z $AMI ]]; then
    echo "Enter AMI name:"
        read AMI
fi
if [[ -z $PEM ]]; then
    echo "Enter name of security key:"
        read PEM
fi
if [[ -z $INSTANCETYPE ]]; then
    echo "Enter instance type:"
        read INSTANCETYPE
fi      
if [[ -z $IP ]]; then
    echo "Enter IP address:"
        read IP
fi
if [[ -z $CIDR ]]; then
    echo "Enter CIDR n the format xxx..xxx.xxx.xxx/yy:"
        read CIDR 
fi
if [[ -z $SGNAME ]]; then
    echo "Enter security group name:"
        read SGNAME
fi
if [[ -z $NAMETAG ]]; then
    echo "Enter name tags for instance and volumes:"
        read NAMETAG
fi

echo "*** 1.  Prequisites checking"
EXISTINGIP=$(aws ec2 describe-network-interfaces --filter Name=private-ip-address,Values=$IP | awk -F\t '/PRIVATEIPADDRESSES/ {print $3}' | grep $IP) 
if [ $EXISTINGIP ]
then
	INSTID=$(aws ec2 describe-network-interfaces --filter Name=private-ip-address,Values=$IP | awk -F\t '/ATTACHMENT/ {print $6}')
	echo "*** 1.1 ERROR: requested IP address $IP is already in use by instance $INSTID. Will stop here..."
	exit 1
else
        echo "*** 1.1 OK: requested IP address $IP is available"
fi
SUBNET=$(aws ec2 describe-subnets --filter  Name=cidrBlock,Values=$CIDR | awk -F\t '/SUBNETS/ {print $8}')
AZ=$(aws ec2 describe-subnets --filter  Name=cidrBlock,Values=$CIDR | awk -F\t '/SUBNETS/ {print $2}')
VPC=$(aws ec2 describe-subnets --filter  Name=cidrBlock,Values=$CIDR | awk -F\t '/SUBNETS/ {print $9}')
if [ $SUBNET ]
then
        echo "*** 1.2 OK: requested CIDR $CIDR belongs to $SUBNET in $AZ , $VPC"
else
	echo "*** 1.2 ERROR: no subnet found for CIDR $CIDR . Will stop here..."
	exit 1
fi
echo "*** 1.3 Warning: No check whether $IP fits into CIDR $CIDR !"
SECURITY=$(aws ec2 describe-security-groups --filters Name=group-name,Values=${SGNAME} | awk -F\t '/SECURITYGROUPS/ {print $3}')
SECURITYTEXT=$(aws ec2 describe-security-groups --filters Name=group-name,Values=${SGNAME} | awk -F\t '/SECURITYGROUPS/ {print $2}')
if [ $SECURITY ]
then
        echo "*** 1.4 OK: requested security group $SGNAME is $SECURITY ($SECURITYTEXT)"
else
        echo "*** 1.4 ERROR: requested security group $SGNAME not found. Will stop here"
	exit 1
fi
PEMRESULT=$(aws ec2 describe-key-pairs --filters Name=key-name,Values=$PEM| awk -F\t '/KEYPAIRS/ {print $3}')
if [ $PEMRESULT ]
then
        echo "*** 1.5 OK: requested PEM $PEM exists."
else
        echo "*** 1.5 ERROR: requested PEM $PEM not found. Will stop here"
	exit 1
fi
AMINAME=$(aws ec2 describe-images --image-ids $AMI | awk -F\t '/IMAGES/ {print $6}')
if [ $AMINAME ]
then
        echo "*** 1.6 OK: AMI $AMI exists ($AMINAME)"
else
        echo "*** 1.6 ERROR: AMI $AMI does not exist. Will stop here"
	exit 1
fi
echo -n "Do you want to create this instance? (y/n) "
old_stty_cfg=$(stty -g)
stty raw -echo ; answer=$(head -c 1) ; stty $old_stty_cfg # Care playing with stty
if echo "$answer" | grep -iq "^y" ;then
    echo Yes
else
    echo No
	exit
fi
echo "*** 2. About to create the instance"
ID=$(aws ec2 run-instances \
	--key-name $PEM \
	--instance-type $INSTANCETYPE \
        --count 1 \
	--block-device-mappings file://disks.json \
	--image-id $AMI \
	--monitoring Enabled=true \
	--instance-initiated-shutdown-behavior stop \
	--security-group-ids $SECURITY \
	--subnet-id $SUBNET \
	--private-ip-address $IP \
	--ebs-optimized  | \
	awk '/INSTANCES/ {print $8}' \
) 
echo "*** 2.1 Created system with Id: $ID"
aws ec2 create-tags --resources $ID  --tags Key=Name,Value=${NAMETAG}
echo "*** 2.2 Tagged system with Id: $ID with Name: $NAMETAG "
#echo "*** 2.3.0  will sleep for 2s before tagging the volumes with $NAMETAG "
#sleep 2
aws ec2 describe-instances --instance-ids $ID | awk '/EBS/ {print "aws ec2 create-tags --resources " $5 " --tags Key=Name,Value='"$NAMETAG"'" }' | bash -
echo "*** 2.3.1 Tagged all volumes from system with Id: $ID with Name: $NAMETAG "
echo "*** 2.4 The created instance: $ID with Name: $NAMETAG "
aws ec2 describe-instances --instance-ids $ID
